@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	24 May 1995
- * Last Edited:	26 November 2003
+ * Last Edited:	6 December 2004
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2003 University of Washington.
+ * Copyright 1988-2004 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -491,11 +491,9 @@
 #define smtp_auth s_auth
 #define smtp_challenge s_chal
 #define smtp_close s_clos
-#define smtp_debug s_dbug
 #define smtp_ehlo s_ehlo
 #define smtp_fake s_fake
 #define smtp_mail s_mail
-#define smtp_nodebug s_nodb
 #define smtp_open_full s_open
 #define smtp_rcpt s_rcpt
 #define smtp_reply s_repl
@@ -528,6 +526,7 @@
 #define textcpystring txcpst
 #define textcpyoffstring txcpos
 #define utf8_cstext u8_cst
+#define utf8_cstocstext u8_cct
 #define utf8_get u8_get
 #define utf8_iso2022text u8_i22
 #define utf8_mime2text u8_mi2
