@@ -10,14 +10,15 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	20 December 1989
- * Last Edited:	20 December 2002
+ * Last Edited:	4 November 2004
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2002 University of Washington.
+ * Copyright 1988-2004 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
-
+
+
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
@@ -27,12 +28,260 @@ extern int errno;		/* just in case */
 #include "osdep.h"
 #include <time.h>
 #include <sys/stat.h>
-#include "mmdf.h"
 #include "pseudo.h"
 #include "fdstring.h"
 #include "misc.h"
 #include "dummy.h"
 
+/* Supposedly, this page has everything the MMDF driver needs to know about
+ * the MMDF delimiter.  By changing these macros, the MMDF driver should
+ * change with it.  Note that if you change the length of MMDFHDRTXT you
+ * also need to change the ISMMDF and RETIFMMDFWRD macros to reflect the new
+ * size.
+ */
+
+
+/* Useful MMDF constants */
+
+#define MMDFCHR '\01'		/* MMDF character */
+#define MMDFCHRS 0x01010101	/* MMDF header character spread in a word */
+				/* MMDF header text */
+#define MMDFHDRTXT "\01\01\01\01\n"
+				/* length of MMDF header text */
+#define MMDFHDRLEN (sizeof (MMDFHDRTXT) - 1)
+
+
+/* Validate MMDF header
+ * Accepts: pointer to candidate string to validate as an MMDF header
+ * Returns: T if valid; else NIL
+ */
+
+#define ISMMDF(s)							\
+  ((*(s) == MMDFCHR) && ((s)[1] == MMDFCHR) && ((s)[2] == MMDFCHR) &&	\
+   ((s)[3] == MMDFCHR) && ((s)[4] == '\n'))
+
+
+/* Return if a 32-bit word has the start of an MMDF header
+ * Accepts: pointer to word of four bytes to validate as an MMDF header
+ * Returns: pointer to MMDF header, else proceeds
+ */
+
+#define RETIFMMDFWRD(s) {						\
+  if (s[3] == MMDFCHR) {						\
+    if ((s[4] == MMDFCHR) && (s[5] == MMDFCHR) && (s[6] == MMDFCHR) &&	\
+	(s[7] == '\n')) return s + 3;					\
+    else if (s[2] == MMDFCHR) {						\
+      if ((s[4] == MMDFCHR) && (s[5] == MMDFCHR) && (s[6] == '\n'))	\
+	return s + 2;							\
+      else if (s[1] == MMDFCHR) {					\
+	if ((s[4] == MMDFCHR) && (s[5] == '\n')) return s + 1;		\
+	else if ((*s == MMDFCHR) && (s[4] == '\n')) return s;		\
+      }									\
+    }									\
+  }									\
+}
+
+/* Validate line
+ * Accepts: pointer to candidate string to validate as a From header
+ *	    return pointer to end of date/time field
+ *	    return pointer to offset from t of time (hours of ``mmm dd hh:mm'')
+ *	    return pointer to offset from t of time zone (if non-zero)
+ * Returns: t,ti,zn set if valid From string, else ti is NIL
+ */
+
+#define VALID(s,x,ti,zn) {						\
+  ti = 0;								\
+  if ((*s == 'F') && (s[1] == 'r') && (s[2] == 'o') && (s[3] == 'm') &&	\
+      (s[4] == ' ')) {							\
+    for (x = s + 5; *x && *x != '\n'; x++);				\
+    if (*x) {								\
+      if (x - s >= 41) {						\
+	for (zn = -1; x[zn] != ' '; zn--);				\
+	if ((x[zn-1] == 'm') && (x[zn-2] == 'o') && (x[zn-3] == 'r') &&	\
+	    (x[zn-4] == 'f') && (x[zn-5] == ' ') && (x[zn-6] == 'e') &&	\
+	    (x[zn-7] == 't') && (x[zn-8] == 'o') && (x[zn-9] == 'm') &&	\
+	    (x[zn-10] == 'e') && (x[zn-11] == 'r') && (x[zn-12] == ' '))\
+	  x += zn - 12;							\
+      }									\
+      if (x - s >= 27) {						\
+	if (x[-5] == ' ') {						\
+	  if (x[-8] == ':') zn = 0,ti = -5;				\
+	  else if (x[-9] == ' ') ti = zn = -9;				\
+	  else if ((x[-11] == ' ') && ((x[-10]=='+') || (x[-10]=='-')))	\
+	    ti = zn = -11;						\
+	}								\
+	else if (x[-4] == ' ') {					\
+	  if (x[-9] == ' ') zn = -4,ti = -9;				\
+	}								\
+	else if (x[-6] == ' ') {					\
+	  if ((x[-11] == ' ') && ((x[-5] == '+') || (x[-5] == '-')))	\
+	    zn = -6,ti = -11;						\
+	}								\
+	if (ti && !((x[ti - 3] == ':') &&				\
+		    (x[ti -= ((x[ti - 6] == ':') ? 9 : 6)] == ' ') &&	\
+		    (x[ti - 3] == ' ') && (x[ti - 7] == ' ') &&		\
+		    (x[ti - 11] == ' '))) ti = 0;			\
+      }									\
+    }									\
+  }									\
+}
+
+/* You are not expected to understand this macro, but read the next page if
+ * you are not faint of heart.
+ *
+ * Known formats to the VALID macro are:
+ *		From user Wed Dec  2 05:53 1992
+ * BSD		From user Wed Dec  2 05:53:22 1992
+ * SysV		From user Wed Dec  2 05:53 PST 1992
+ * rn		From user Wed Dec  2 05:53:22 PST 1992
+ *		From user Wed Dec  2 05:53 -0700 1992
+ * emacs	From user Wed Dec  2 05:53:22 -0700 1992
+ *		From user Wed Dec  2 05:53 1992 PST
+ *		From user Wed Dec  2 05:53:22 1992 PST
+ *		From user Wed Dec  2 05:53 1992 -0700
+ * Solaris	From user Wed Dec  2 05:53:22 1992 -0700
+ *
+ * Plus all of the above with `` remote from xxx'' after it. Thank you very
+ * much, smail and Solaris, for making my life considerably more complicated.
+ */
+
+/*
+ * What?  You want to understand the VALID macro anyway?  Alright, since you
+ * insist.  Actually, it isn't really all that difficult, provided that you
+ * take it step by step.
+ *
+ * Line 1	Initializes the return ti value to failure (0);
+ * Lines 2-3	Validates that the 1st-5th characters are ``From ''.
+ * Lines 4-5	Validates that there is an end of line and points x at it.
+ * Lines 6-13	First checks to see if the line is at least 41 characters long.
+ *		If so, it scans backwards to find the rightmost space.  From
+ *		that point, it scans backwards to see if the string matches
+ *		`` remote from''.  If so, it sets x to point to the space at
+ *		the start of the string.
+ * Line 14	Makes sure that there are at least 27 characters in the line.
+ * Lines 15-20	Checks if the date/time ends with the year (there is a space
+ *		five characters back).  If there is a colon three characters
+ *		further back, there is no timezone field, so zn is set to 0
+ *		and ti is set in front of the year.  Otherwise, there must
+ *		either to be a space four characters back for a three-letter
+ *		timezone, or a space six characters back followed by a + or -
+ *		for a numeric timezone; in either case, zn and ti become the
+ *		offset of the space immediately before it.
+ * Lines 21-23	Are the failure case for line 14.  If there is a space four
+ *		characters back, it is a three-letter timezone; there must be a
+ *		space for the year nine characters back.  zn is the zone
+ *		offset; ti is the offset of the space.
+ * Lines 24-27	Are the failure case for line 20.  If there is a space six
+ *		characters back, it is a numeric timezone; there must be a
+ *		space eleven characters back and a + or - five characters back.
+ *		zn is the zone offset; ti is the offset of the space.
+ * Line 28-31	If ti is valid, make sure that the string before ti is of the
+ *		form www mmm dd hh:mm or www mmm dd hh:mm:ss, otherwise
+ *		invalidate ti.  There must be a colon three characters back
+ *		and a space six or nine	characters back (depending upon
+ *		whether or not the character six characters back is a colon).
+ *		There must be a space three characters further back (in front
+ *		of the day), one seven characters back (in front of the month),
+ *		and one eleven characters back (in front of the day of week).
+ *		ti is set to be the offset of the space before the time.
+ *
+ * Why a macro?  It gets invoked a *lot* in a tight loop.  On some of the
+ * newer pipelined machines it is faster being open-coded than it would be if
+ * subroutines are called.
+ *
+ * Why does it scan backwards from the end of the line, instead of doing the
+ * much easier forward scan?  There is no deterministic way to parse the
+ * ``user'' field, because it may contain unquoted spaces!  Yes, I tested it to
+ * see if unquoted spaces were possible.  They are, and I've encountered enough
+ * evil mail to be totally unwilling to trust that ``it will never happen''.
+ */
+
+/* Build parameters */
+
+#define KODRETRY 15		/* kiss-of-death retry in seconds */
+#define LOCKTIMEOUT 5		/* lock timeout in minutes */
+#define CHUNK 16384		/* read-in chunk size */
+
+
+/* MMDF I/O stream local data */
+
+typedef struct mmdf_local {
+  unsigned int dirty : 1;	/* disk copy needs updating */
+  unsigned int pseudo : 1;	/* uses a pseudo message */
+  int fd;			/* mailbox file descriptor */
+  int ld;			/* lock file descriptor */
+  char *lname;			/* lock file name */
+  off_t filesize;		/* file size parsed */
+  time_t filetime;		/* last file time */
+  unsigned char *buf;		/* temporary buffer */
+  unsigned long buflen;		/* current size of temporary buffer */
+  unsigned long uid;		/* current text uid */
+  SIZEDTEXT text;		/* current text */
+  unsigned long textlen;	/* current text length */
+  char *line;			/* returned line */
+} MMDFLOCAL;
+
+
+/* Convenient access to local data */
+
+#define LOCAL ((MMDFLOCAL *) stream->local)
+
+
+/* MMDF protected file structure */
+
+typedef struct mmdf_file {
+  MAILSTREAM *stream;		/* current stream */
+  off_t curpos;			/* current file position */
+  off_t protect;		/* protected position */
+  off_t filepos;		/* current last written file position */
+  char *buf;			/* overflow buffer */
+  size_t buflen;		/* current overflow buffer length */
+  char *bufpos;			/* current buffer position */
+} MMDFFILE;
+
+/* Function prototypes */
+
+DRIVER *mmdf_valid (char *name);
+long mmdf_isvalid (char *name,char *tmp);
+long mmdf_isvalid_fd (int fd,char *tmp);
+void *mmdf_parameters (long function,void *value);
+void mmdf_scan (MAILSTREAM *stream,char *ref,char *pat,char *contents);
+void mmdf_list (MAILSTREAM *stream,char *ref,char *pat);
+void mmdf_lsub (MAILSTREAM *stream,char *ref,char *pat);
+long mmdf_create (MAILSTREAM *stream,char *mailbox);
+long mmdf_delete (MAILSTREAM *stream,char *mailbox);
+long mmdf_rename (MAILSTREAM *stream,char *old,char *newname);
+MAILSTREAM *mmdf_open (MAILSTREAM *stream);
+void mmdf_close (MAILSTREAM *stream,long options);
+char *mmdf_header (MAILSTREAM *stream,unsigned long msgno,
+		   unsigned long *length,long flags);
+long mmdf_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags);
+char *mmdf_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
+		      unsigned long *length,long flags);
+void mmdf_flagmsg (MAILSTREAM *stream,MESSAGECACHE *elt);
+long mmdf_ping (MAILSTREAM *stream);
+void mmdf_check (MAILSTREAM *stream);
+void mmdf_check (MAILSTREAM *stream);
+void mmdf_expunge (MAILSTREAM *stream);
+long mmdf_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options);
+long mmdf_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data);
+int mmdf_append_msg (MAILSTREAM *stream,FILE *sf,char *flags,char *date,
+		     STRING *msg);
+
+void mmdf_abort (MAILSTREAM *stream);
+char *mmdf_file (char *dst,char *name);
+int mmdf_lock (char *file,int flags,int mode,DOTLOCK *lock,int op);
+void mmdf_unlock (int fd,MAILSTREAM *stream,DOTLOCK *lock);
+int mmdf_parse (MAILSTREAM *stream,DOTLOCK *lock,int op);
+char *mmdf_mbxline (MAILSTREAM *stream,STRING *bs,unsigned long *size);
+unsigned long mmdf_pseudo (MAILSTREAM *stream,char *hdr);
+unsigned long mmdf_xstatus (MAILSTREAM *stream,char *status,MESSAGECACHE *elt,
+			    long flag);
+long mmdf_rewrite (MAILSTREAM *stream,unsigned long *nexp,DOTLOCK *lock);
+long mmdf_extend (MAILSTREAM *stream,unsigned long size);
+void mmdf_write (MMDFFILE *f,char *s,unsigned long i);
+void mmdf_phys_write (MMDFFILE *f,char *buf,size_t size);
+
 /* MMDF mail routines */
 
 
@@ -116,9 +365,12 @@ long mmdf_isvalid (char *name,char *tmp)
 				/* error -1 for invalid format */
       if (!(ret = mmdf_isvalid_fd (fd,tmp))) errno = -1;
       close (fd);		/* close the file */
-      tp[0] = sbuf.st_atime;	/* preserve atime and mtime */
-      tp[1] = sbuf.st_mtime;
-      utime (file,tp);		/* set the times */
+				/* \Marked status? */
+      if ((sbuf.st_ctime > sbuf.st_atime) || (sbuf.st_mtime > sbuf.st_atime)) {
+	tp[0] = sbuf.st_atime;	/* preserve atime and mtime */
+	tp[1] = sbuf.st_mtime;
+	utime (file,tp);	/* set the times */
+      }
     }
   }
   return ret;			/* return what we should */
@@ -147,7 +399,13 @@ long mmdf_isvalid_fd (int fd,char *tmp)
 
 void *mmdf_parameters (long function,void *value)
 {
-  return NIL;
+  void *ret = NIL;
+  switch ((int) function) {
+  case GET_INBOXPATH:
+    if (value) ret = dummy_file ((char *) value,"INBOX");
+    break;
+  }
+  return ret;
 }
 
 /* MMDF mail scan mailboxes
@@ -267,15 +525,19 @@ long mmdf_rename (MAILSTREAM *stream,char *old,char *newname)
   long i;
   struct stat sbuf;
   MM_CRITICAL (stream);	/* get the c-client lock */
-  if (newname && !((s = dummy_file (tmp,newname)) && *s))
-    sprintf (tmp,"Can't rename mailbox %.80s to %.80s: invalid name",
+  if (!dummy_file (file,old) ||
+      (newname && !((s = mailboxfile (tmp,newname)) && *s)))
+    sprintf (tmp,newname ?
+	     "Can't rename mailbox %.80s to %.80s: invalid name" :
+	     "Can't delete mailbox %.80s: invalid name",
 	     old,newname);
 				/* lock out other c-clients */
-  else if ((ld = lockname (lock,dummy_file (file,old),LOCK_EX|LOCK_NB,&i)) < 0)
+  else if ((ld = lockname (lock,file,LOCK_EX|LOCK_NB,&i)) < 0)
     sprintf (tmp,"Mailbox %.80s is in use by another process",old);
+
   else {
     if ((fd = mmdf_lock (file,O_RDWR,S_IREAD|S_IWRITE,&lockx,LOCK_EX)) < 0)
-      sprintf (tmp,"Can't lock mailbox %.80s: %s",old,strerror (errno));
+       sprintf (tmp,"Can't lock mailbox %.80s: %s",old,strerror (errno));
     else {
       if (newname) {		/* want rename? */
 				/* found superior to destination name? */
@@ -331,13 +593,19 @@ MAILSTREAM *mmdf_open (MAILSTREAM *stream)
 				/* note if an INBOX or not */
   stream->inbox = !compare_cstring (stream->mailbox,"INBOX");
 				/* canonicalize the stream mailbox name */
-  dummy_file (tmp,stream->mailbox);
+  if (!dummy_file (tmp,stream->mailbox)) {
+    sprintf (tmp,"Can't open - invalid name: %.80s",stream->mailbox);
+    MM_LOG (tmp,ERROR);
+    return NIL;
+  }
 				/* flush old name */
   fs_give ((void **) &stream->mailbox);
 				/* save canonical name */
   stream->mailbox = cpystr (tmp);
   LOCAL->fd = LOCAL->ld = -1;	/* no file or state locking yet */
   LOCAL->buf = (char *) fs_get ((LOCAL->buflen = CHUNK) + 1);
+  LOCAL->text.data = (unsigned char *)
+    fs_get ((LOCAL->text.size = MAXMESSAGESIZE) + 1);
   stream->sequence++;		/* bump sequence number */
 
 				/* make lock for read/write access */
@@ -444,7 +712,7 @@ char *mmdf_header (MAILSTREAM *stream,unsigned long msgno,
 		   unsigned long *length,long flags)
 {
   MESSAGECACHE *elt;
-  char *s,*t,*tl;
+  unsigned char *s,*t,*tl;
   *length = 0;			/* default to empty */
   if (flags & FT_UID) return "";/* UID call "impossible" */
   elt = mail_elt (stream,msgno);/* get cache */
@@ -540,7 +808,7 @@ char *mmdf_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
 {
   FDDATA d;
   STRING bs;
-  char *s,*t,*tl,tmp[CHUNK];
+  unsigned char *s,*t,*tl,tmp[CHUNK];
 				/* go to text position */
   lseek (LOCAL->fd,elt->private.special.offset +
 	 elt->private.msg.text.offset,L_SET);
@@ -559,20 +827,26 @@ char *mmdf_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
       if ((*t != '\r') || (t[1] != '\n')) *s++ = *t;
 				/* adjust length */
     LOCAL->buf[*length = s - LOCAL->buf - 1] = '\0';
+    return LOCAL->buf;
   }
 
-  else {			/* need to make a CRLF version */
-    if (elt->rfc822_size > LOCAL->buflen) {
+				/* have it cached already? */
+  if (elt->private.uid != LOCAL->uid) {
+				/* not cached, cache it now */
+    LOCAL->uid = elt->private.uid;
+				/* is buffer big enough? */
+    if (elt->rfc822_size > LOCAL->text.size) {
       /* excessively conservative, but the right thing is too hard to do */
-      fs_give ((void **) &LOCAL->buf);
-      LOCAL->buf = (char *) fs_get ((LOCAL->buflen = elt->rfc822_size) + 1);
+      fs_give ((void **) &LOCAL->text.data);
+      LOCAL->text.data = (unsigned char *)
+	fs_get ((LOCAL->text.size = elt->rfc822_size) + 1);
     }
     d.fd = LOCAL->fd;		/* yes, set up file descriptor */
     d.pos = elt->private.special.offset + elt->private.msg.text.offset;
     d.chunk = tmp;		/* initial buffer chunk */
     d.chunksize = CHUNK;	/* file chunk size */
     INIT (&bs,fd_string,&d,elt->private.msg.text.text.size);
-    for (s = LOCAL->buf; SIZE (&bs);) switch (CHR (&bs)) {
+    for (s = (char *) LOCAL->text.data; SIZE (&bs);) switch (CHR (&bs)) {
     case '\r':			/* carriage return seen */
       *s++ = SNX (&bs);		/* copy it and any succeeding LF */
       if (SIZE (&bs) && (CHR (&bs) == '\n')) *s++ = SNX (&bs);
@@ -583,9 +857,11 @@ char *mmdf_text_work (MAILSTREAM *stream,MESSAGECACHE *elt,
       *s++ = SNX (&bs);		/* copy characters */
     }
     *s = '\0';			/* tie off buffer */
-    *length = s - LOCAL->buf;	/* calculate length */
+				/* calculate length of cached data */
+    LOCAL->textlen = s - LOCAL->text.data;
   }
-  return LOCAL->buf;
+  *length = LOCAL->textlen;	/* return from cache */
+  return (char *) LOCAL->text.data;
 }
 
 /* MMDF per-message modify flag
@@ -720,11 +996,7 @@ long mmdf_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
 				/* make sure valid mailbox */
   if (!mmdf_isvalid (mailbox,file)) switch (errno) {
   case ENOENT:			/* no such file? */
-    if (((mailbox[0] == 'I') || (mailbox[0] == 'i')) &&
-	((mailbox[1] == 'N') || (mailbox[1] == 'n')) &&
-	((mailbox[2] == 'B') || (mailbox[2] == 'b')) &&
-	((mailbox[3] == 'O') || (mailbox[3] == 'o')) &&
-	((mailbox[4] == 'X') || (mailbox[4] == 'x')) && !mailbox[5]) {
+    if (!compare_cstring (mailbox,"INBOX")) {
       if (pc) return (*pc) (stream,sequence,mailbox,options);
       mmdf_create (NIL,"INBOX");/* create empty INBOX */
       break;
@@ -782,8 +1054,11 @@ long mmdf_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
     ftruncate (fd,sbuf.st_size);
     ret = NIL;
   }
-  tp[0] = sbuf.st_atime;	/* preserve atime */
   tp[1] = time (0);		/* set mtime to now */
+  if (ret) tp[0] = tp[1] - 1;	/* set atime to now-1 if successful copy */
+  else tp[0] =			/* else preserve \Marked status */
+	 ((sbuf.st_ctime > sbuf.st_atime) || (sbuf.st_mtime > sbuf.st_atime)) ?
+	 sbuf.st_atime : tp[1];
   utime (file,tp);		/* set the times */
   mmdf_unlock (fd,NIL,&lock);	/* unlock and close mailbox */
   MM_NOCRITICAL (stream);	/* release critical */
@@ -819,15 +1094,16 @@ long mmdf_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   STRING *message;
   long ret = LONGT;
 				/* default stream to prototype */
-  if (!stream) stream = user_flags (&mmdfproto);
+  if (!stream) {		/* stream specified? */
+    stream = &mmdfproto;	/* no, default stream to prototype */
+    for (i = 0; i < NUSERFLAGS && stream->user_flags[i]; ++i)
+      fs_give ((void **) &stream->user_flags[i]);
+    stream->kwd_create = T;	/* allow new flags */
+  }
 				/* make sure valid mailbox */
   if (!mmdf_valid (mailbox)) switch (errno) {
   case ENOENT:			/* no such file? */
-    if (((mailbox[0] == 'I') || (mailbox[0] == 'i')) &&
-	((mailbox[1] == 'N') || (mailbox[1] == 'n')) &&
-	((mailbox[2] == 'B') || (mailbox[2] == 'b')) &&
-	((mailbox[3] == 'O') || (mailbox[3] == 'o')) &&
-	((mailbox[4] == 'X') || (mailbox[4] == 'x')) && !mailbox[5]) {
+    if (!compare_cstring (mailbox,"INBOX")) {
       mmdf_create (NIL,"INBOX");/* create empty INBOX */
       break;
     }
@@ -901,16 +1177,18 @@ long mmdf_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   for (; i && ((j = fread (buf,1,min ((long) BUFLEN,i),sf)) &&
 	       (fwrite (buf,1,j,df) == j)); i -= j);
   fclose (sf);			/* done with scratch file */
-  tp[0] = sbuf.st_atime;	/* preserve atime */
+  tp[1] = time (0);		/* set mtime to now */
 				/* make sure append wins, fsync() necessary */
   if (i || (fflush (df) == EOF) || fsync (fd)) {
     sprintf (buf,"Message append failed: %s",strerror (errno));
     MM_LOG (buf,ERROR);
     ftruncate (fd,sbuf.st_size);
-    tp[1] = sbuf.st_mtime;	/* preserve mtime */
+    tp[0] =			/* preserve \Marked status */
+      ((sbuf.st_ctime > sbuf.st_atime) || (sbuf.st_mtime > sbuf.st_atime)) ?
+      sbuf.st_atime : tp[1];
     ret = NIL;			/* return error */
   }
-  else tp[1] = time (0);	/* set mtime to now */
+  else tp[0] = tp[1] - 1;	/* set atime to now-1 if successful copy */
   utime (file,tp);		/* set the times */
   mmdf_unlock (fd,NIL,&lock);	/* unlock and close mailbox */
   fclose (df);
@@ -929,8 +1207,10 @@ long mmdf_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 int mmdf_append_msg (MAILSTREAM *stream,FILE *sf,char *flags,char *date,
 		     STRING *msg)
 {
-  unsigned long uf;
+  unsigned long i,uf;
   int c;
+  char tmp[MAILTMPLEN];
+  int hdrp = T;
   long f = mail_parse_flags (stream,flags,&uf);
 				/* build initial header */
   if ((fprintf (sf,"%sFrom %s@%s %sStatus: ",
@@ -947,10 +1227,51 @@ int mmdf_append_msg (MAILSTREAM *stream,FILE *sf,char *flags,char *date,
       return NIL;
 				/* tie off flags */
   if (putc ('\n',sf) == EOF) return NIL;
-  while (SIZE (msg)) {		/* copy text, flush CTRL/A and CR from CRLF */
-    if (((c = 0xff & SNX (msg)) != MMDFCHR) &&
-	(((c == '\r') && SIZE (msg) && ((c = SNX (msg)) != '\n') &&
-	  (putc ('\r',sf) == EOF)) || (putc (c,sf) == EOF))) return NIL;
+  while (SIZE (msg)) {		/* copy text to scratch file */
+				/* disregard CRs */
+    while ((c = (SIZE (msg)) ? (c = 0xff & SNX (msg)) : '\n') == '\r');
+				/* see if line needs special treatment */
+    if (hdrp && ((c == 'S') || (c == 'X'))) {
+				/* copy line to buffer */
+      for (i = 1,tmp[0] = c; (c != '\n') && (i < MAILTMPLEN); )
+	if ((c = (SIZE (msg)) ? (0xff & SNX (msg)) : '\n') != '\r')
+	  tmp[i++] = c;
+				/* insert X- before metadata header */
+      if ((((i > 6) && (tmp[0] == 'S') && (tmp[1] == 't') &&
+	    (tmp[2] == 'a') && (tmp[3] == 't') && (tmp[4] == 'u') &&
+	    (tmp[5] == 's') && (tmp[6] == ':')) ||
+	   (((i > 5) && (tmp[0] == 'X') && (tmp[1] == '-')) &&
+	    (((tmp[2] == 'U') && (tmp[3] == 'I') && (tmp[4] == 'D') &&
+	      (tmp[5] == ':')) ||
+	     ((i > 6) && (tmp[2] == 'I') && (tmp[3] == 'M') &&
+	      (tmp[4] == 'A') && (tmp[5] == 'P') &&
+	      ((tmp[6] == ':') ||
+	       ((i > 10) && (tmp[6] == 'b') && (tmp[7] == 'a') &&
+		(tmp[8] == 's') && (tmp[9] == 'e') && (tmp[10] == ':')))) ||
+	     ((i > 8) && (tmp[2] == 'S') && (tmp[3] == 't') &&
+	      (tmp[4] == 'a') && (tmp[5] == 't') && (tmp[6] == 'u') &&
+	      (tmp[7] == 's') && (tmp[8] == ':')) ||
+	     ((i > 10) && (tmp[2] == 'K') && (tmp[3] == 'e') &&
+	      (tmp[4] == 'y') && (tmp[5] == 'w') && (tmp[6] == 'o') &&
+	      (tmp[7] == 'r') && (tmp[8] == 'd') && (tmp[9] == 's') &&
+	      (tmp[10] == ':'))))) && (fputs ("X-Original-",sf) == EOF))
+	return NIL;
+				/* write buffered text */
+      if (fwrite (tmp,1,i,sf) != i) return NIL;
+				/* set up to copy remainder of line */
+      if ((c != '\n') && SIZE (msg)) c = 0xff & SNX (msg);
+      else continue;		/* end of line or end of message */
+    }
+				/* check for end of header */
+    else if (hdrp && (c == '\n')) hdrp = NIL;
+    do switch (c) {		/* copy line */
+    case MMDFCHR:		/* flush CTRL/A */
+    case '\r':			/* and CR */
+      break;
+    default:			/* any other character */
+      if (putc (c,sf) == EOF) return NIL;
+    }
+    while ((c != '\n') && SIZE (msg) && ((c = 0xff & SNX (msg)) ? c : T));
   }
 				/* write trailer and return */
   return (fputs (mmdfhdr,sf) == EOF) ? NIL : T;
@@ -975,6 +1296,7 @@ void mmdf_abort (MAILSTREAM *stream)
     if (LOCAL->lname) fs_give ((void **) &LOCAL->lname);
 				/* free local text buffers */
     if (LOCAL->buf) fs_give ((void **) &LOCAL->buf);
+    if (LOCAL->text.data) fs_give ((void **) &LOCAL->text.data);
     if (LOCAL->line) fs_give ((void **) &LOCAL->line);
 				/* nuke the local data */
     fs_give ((void **) &stream->local);
@@ -1023,16 +1345,33 @@ int mmdf_lock (char *file,int flags,int mode,DOTLOCK *lock,int op)
 
 void mmdf_unlock (int fd,MAILSTREAM *stream,DOTLOCK *lock)
 {
-  struct stat sbuf;
-  time_t tp[2];
-  fstat (fd,&sbuf);		/* get file times */
-				/* if stream and csh would think new mail */
-  if (stream && (sbuf.st_atime <= sbuf.st_mtime)) {
-    tp[0] = time (0);		/* set atime to now */
+  if (stream) {			/* need to muck with times? */
+    struct stat sbuf;
+    time_t tp[2];
+    time_t now = time (0);
+    fstat (fd,&sbuf);		/* get file times */
+    if (LOCAL->ld >= 0) {	/* yes, readwrite session? */
+      tp[0] = now;		/* set atime to now */
 				/* set mtime to (now - 1) if necessary */
-    tp[1] = tp[0] > sbuf.st_mtime ? sbuf.st_mtime : tp[0] - 1;
+      tp[1] = (now > sbuf.st_mtime) ? sbuf.st_mtime : now - 1;
+    }
+    else if (stream->recent) {	/* readonly with recent messages */
+      if ((sbuf.st_atime >= sbuf.st_mtime) ||
+	  (sbuf.st_atime >= sbuf.st_ctime))
+				/* keep past mtime, whack back atime */
+	tp[0] = (tp[1] = (sbuf.st_mtime < now) ? sbuf.st_mtime : now) - 1;
+      else now = 0;		/* no time change needed */
+    }
+				/* readonly with no recent messages */
+    else if ((sbuf.st_atime < sbuf.st_mtime) ||
+	     (sbuf.st_atime < sbuf.st_ctime)) {
+      tp[0] = now;		/* set atime to now */
+				/* set mtime to (now - 1) if necessary */
+      tp[1] = (now > sbuf.st_mtime) ? sbuf.st_mtime : now - 1;
+    }
+    else now = 0;		/* no time change needed */
 				/* set the times, note change */
-    if (!utime (stream->mailbox,tp)) LOCAL->filetime = tp[1];
+    if (now && !utime (stream->mailbox,tp)) LOCAL->filetime = tp[1];
   }
   flock (fd,LOCK_UN);		/* release flock'ers */
   if (!stream) close (fd);	/* close the file if no stream */
@@ -1050,7 +1389,7 @@ int mmdf_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 {
   int ti,zn,m;
   unsigned long i,j,k;
-  char c,*s,*t,*u,tmp[MAILTMPLEN],date[30];
+  unsigned char c,*s,*t,*u,tmp[MAILTMPLEN],date[30];
   int retain = T;
   unsigned long nmsgs = stream->nmsgs;
   unsigned long prevuid = nmsgs ? mail_elt (stream,nmsgs)->private.uid : 0;
@@ -1106,7 +1445,7 @@ int mmdf_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
       do {			/* read MMDF header */
 	if (!(i && ISMMDF (s))){/* see if valid MMDF header */
 	  sprintf (tmp,"Unexpected changes to mailbox (try restarting): %.20s",
-		   s);
+		   (char *) s);
 				/* see if we can back up to a line */
 	  if (i && (j > MMDFHDRLEN)) {
 	    SETPOS (&bs,j -= MMDFHDRLEN);
@@ -1160,7 +1499,7 @@ int mmdf_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 	  date[7] = t[ti + 1]; date[8] = t[ti + 2];
 	  date[9] = t[ti + 3]; date[10] = t[ti + 4];
 				/* zzz */
-	  t = zn ? (t + zn + 1) : "LCL";
+	  t = zn ? (t + zn + 1) : (unsigned char *) "LCL";
 	  date[21] = *t++; date[22] = *t++; date[23] = *t++;
 	  if ((date[21] != '+') && (date[21] != '-')) date[24] = '\0';
 	  else {		/* numeric time zone */
@@ -1169,7 +1508,7 @@ int mmdf_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 	  }
 				/* set internal date */
 	  if (!mail_parse_date (elt,date)) {
-	    sprintf (tmp,"Unable to parse internal date: %s",date);
+	    sprintf (tmp,"Unable to parse internal date: %s",(char *) date);
 	    MM_LOG (tmp,WARN);
 	  }
 	}
@@ -1359,7 +1698,7 @@ int mmdf_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 	  default:		/* ordinary header line */
 	    if ((*s == 'S') || (*s == 's') ||
 		(((*s == 'X') || (*s == 'x')) && (s[1] == '-'))) {
-	      char *e,*v;
+	      unsigned char *e,*v;
 				/* must match what mail_filter() does */
 	      for (u = s,v = tmp,e = u + min (i,MAILTMPLEN - 1);
 		   (u < e) && ((c = (*u ? *u : (*u = ' '))) != ':') &&
@@ -1376,7 +1715,7 @@ int mmdf_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 		  !compare_cstring (tmp,"X-IMAPBASE")) {
 		char err[MAILTMPLEN];
 		sprintf (err,"Discarding bogus %s header in message %lu",
-			 tmp,elt->msgno);
+			 (char *) tmp,elt->msgno);
 		MM_LOG (err,WARN);
 		retain = NIL;	/* don't retain continuation */
 		break;		/* different case or something */
@@ -1395,7 +1734,7 @@ int mmdf_parse (MAILSTREAM *stream,DOTLOCK *lock,int op)
 	    else {
 	      char err[MAILTMPLEN];
 	      sprintf (err,"Discarding bogus continuation in msg %lu: %.80s",
-		      elt->msgno,s);
+		      elt->msgno,(char *) s);
 	      if (u = strpbrk (err,"\r\n")) *u = '\0';
 	      MM_LOG (err,WARN);
 	      break;		/* different case or something */
