@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	4 September 1991
- * Last Edited:	27 April 2004
+ * Last Edited:	8 July 2004
  * 
  * The IMAP toolkit provided in this Distribution is
  * Copyright 1988-2004 University of Washington.
@@ -167,16 +167,7 @@ DRIVER *news_valid (char *name)
 
 void *news_parameters (long function,void *value)
 {
-  switch ((int) function) {
-  case SET_NEWSRC:
-  case GET_NEWSRC:
-    value = env_parameters (function,NIL);
-    break;
-  default:
-    value = NIL;		/* error case */
-    break;
-  }
-  return value;
+  return (function == GET_NEWSRC) ? env_parameters (function,value) : NIL;
 }
 
 
