@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1993
- * Last Edited:	24 October 2000
+ * Last Edited:	20 April 2005
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2000 University of Washington.
+ * Copyright 1988-2005 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -43,3 +43,4 @@ extern int errno;		/* just in case */
 #include "tcp_unix.c"
 #include "gr_waitp.c"
 #include "tz_sv4.c"
+#include "flocklnx.c"
