@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: status.c,v 4.139 2005/04/26 22:14:46 hubert Exp $";
+static char rcsid[] = "$Id: status.c 14081 2005-09-12 22:04:25Z hubert@u.washington.edu $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: status.c,v 4.139 2005/04/26 22:14:46 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-2004 by the University of Washington.
+   1989-2005 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -437,7 +437,7 @@ int
 status_message_remaining()
 {
     if(message_queue){
-	int d = (int)(displayed_time - time(0))
+	int d = (int)(min(displayed_time - time(0), 0))
 					  + message_queue->min_display_time;
 	return((d > 0) ? d : 0);
     }
@@ -558,7 +558,7 @@ display_message(command)
 	    int    diff;
 
 	    now  = time(0);
-	    diff = (int)(displayed_time - now)
+	    diff = (int)(min(displayed_time - now, 0))
 			+ ((command == NO_OP_COMMAND || command == NO_OP_IDLE)
 			    ? message_queue->max_display_time
 			    : message_queue->min_display_time);
@@ -1211,7 +1211,7 @@ want_to(question, dflt, on_ctrl_C, help, flags)
 
     rv = radio_buttons(q2,
 	(ps_global->ttyo->screen_rows > 4) ? - FOOTER_ROWS(ps_global) : -1,
-	yorn, dflt, on_ctrl_C, help, flags);
+	yorn, dflt, on_ctrl_C, help, flags, NULL);
     fs_give((void **)&q2);
 
     return(rv);
@@ -1231,7 +1231,7 @@ one_try_want_to(question, dflt, on_ctrl_C, help, flags)
     sprintf(q2, "%.*s? ", ps_global->ttyo->screen_cols - 6, question);
     rv = radio_buttons(q2,
 	(ps_global->ttyo->screen_rows > 4) ? - FOOTER_ROWS(ps_global) : -1,
-	yorn, dflt, on_ctrl_C, help, flags | RB_ONE_TRY);
+	yorn, dflt, on_ctrl_C, help, flags | RB_ONE_TRY, NULL);
     fs_give((void **)&q2);
 
     return(rv);
@@ -1279,7 +1279,8 @@ off. Character types that are not buttons will result in a beep (unless one_try
 is set).
   ----*/
 int
-radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
+radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags,
+	      passed_in_km_popped)
     char     *prompt;
     int	      line;
     ESCKEY_S *esc_list;
@@ -1287,6 +1288,7 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
     int       on_ctrl_C;
     HelpType  help_text;
     int	      flags;
+    int      *passed_in_km_popped;
 {
     register int     ch, real_line;
     char            *q, *ds = NULL;
@@ -1343,6 +1345,13 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
     }
 #endif
 
+    if(passed_in_km_popped){
+	km_popped = *passed_in_km_popped;
+	if(FOOTER_ROWS(ps_global) == 1 && km_popped){
+	    FOOTER_ROWS(ps_global) = 3;
+	}
+    }
+
     suspend_busy_alarm();
     flush_ordered_messages();		/* show user previous status msgs */
     mark_status_dirty();		/* clear message next display call */
@@ -1709,6 +1718,8 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 	FOOTER_ROWS(ps_global) = 1;
 	clearfooter(ps_global);
 	ps_global->mangled_body = 1;
+	if(passed_in_km_popped)
+	  *passed_in_km_popped = km_popped;	/* return current value */
     }
 
     return(ch);
@@ -1743,11 +1754,12 @@ double_radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text,
     ESCKEY_S *list = NULL, *list1 = NULL, *list2 = NULL;
     int       count, i = 0, j;
     int       v = OTHER_RETURN_VAL, listnum = 0;
+    int       preserve_km_popped_for_radio_buttons = 0;
 
 #ifdef _WINDOWS
     if(mswin_usedialog())
       return(radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C,
-			   help_text, flags));
+			   help_text, flags, NULL));
 #endif
 
     /* check to see if it will all fit in one */
@@ -1760,7 +1772,7 @@ double_radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text,
     
     if(i <= 12)
       return(radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C,
-			   help_text, flags));
+			   help_text, flags, NULL));
 
     /*
      * Won't fit, split it into two lists.
@@ -1787,7 +1799,27 @@ double_radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text,
     list = construct_combined_esclist(list1, list2);
 
     while(v == OTHER_RETURN_VAL){
-	v = radio_buttons(prompt,line,list,dflt,on_ctrl_C,help_text,flags);
+	int prompt_line;
+
+	/*
+	 * There are some layering violations going on with this
+	 * km_popped stuff. The line is passed in from high above us
+	 * normally, but with this mechanism here we're hardwiring the
+	 * knowledge that FOOTER_ROWS is either 1 or 3, and using that
+	 * below. Radio_buttons itself always puts back the correct
+	 * FOOTER_ROWS before exiting and then restores it based on
+	 * the passed in km_popped and line, which is the second
+	 * layering violation. We could fix it but it's probably not
+	 * worth the effort.
+	 */
+	if(line == -1 && preserve_km_popped_for_radio_buttons)
+	  prompt_line = -3;
+	else
+	  prompt_line = line;
+
+	v = radio_buttons(prompt, prompt_line, list, dflt, on_ctrl_C,
+			  help_text,flags,
+			  &preserve_km_popped_for_radio_buttons);
 	if(v == OTHER_RETURN_VAL){
 	    fs_give((void **)&list);
 	    listnum = 1 - listnum;
